@@ -1,3 +1,4 @@
+from itertools import chain
 import os
 import sys
 import time
@@ -12,32 +13,50 @@ import requests
 from . import jupyterhub
 
 import logging
+
 LOG = logging.getLogger(__name__)
 
-DEFAULT_AUTH_TYPE = 'v3token'
-CONF_GROUP = 'chi'
-RESOURCE_API_URL = os.getenv('CHI_RESOURCE_API_URL',
-                             'https://api.chameleoncloud.org')
+DEFAULT_AUTH_TYPE = "v3token"
+CONF_GROUP = "chi"
+RESOURCE_API_URL = os.getenv("CHI_RESOURCE_API_URL", "https://api.chameleoncloud.org")
 
 
 def default_key_name():
-    username = os.getenv('USER')
-    return f'{username}-jupyter' if username else None
+    username = os.getenv("USER")
+    return f"{username}-jupyter" if username else None
 
 
 session_opts = loading.get_session_conf_options()
 adapter_opts = loading.get_adapter_conf_options()
+
 # Settings that do not affect authentication, but are used  to define defaults
 # for other operations.
+#
+# NOTE: any deprecated options must be defined using the underscore form.
 extra_opts = [
-    cfg.StrOpt('key-name', default=default_key_name(), help=(
-        'Name of the SSH keypair to allow access to launched instances')),
-    cfg.StrOpt('image', help=(
-        'Name of disk image to use when launching instances')),
-    cfg.StrOpt('keypair-private-key', help='Path to the SSH private key file'),
-    cfg.StrOpt('keypair-public-key', help='Path to the SSH public key file'),
+    cfg.StrOpt(
+        "keypair-name",
+        default=default_key_name(),
+        deprecated_opts=[cfg.DeprecatedOpt("key_name")],
+        help=("Name of the SSH keypair to allow access to launched instances"),
+    ),
+    cfg.StrOpt("image", help=("Name of disk image to use when launching instances")),
+    cfg.StrOpt("keypair-private-key", help="Path to the SSH private key file"),
+    cfg.StrOpt("keypair-public-key", help="Path to the SSH public key file"),
 ]
 global_options = session_opts + adapter_opts + extra_opts
+global_option_names = [opt.dest for opt in global_options]
+for extra_opt in extra_opts:
+    # Explicitly add deprecated names so we can issue warnings in get/set.
+    # We only have to do this for our extra_opts, as that's the API we are
+    # advertising/maintain.
+    global_option_names.extend([o.name for o in extra_opt.deprecated_opts])
+
+deprecated_extra_opts = {
+    deprecated_opt.name: current_opt.dest
+    for current_opt in extra_opts
+    for deprecated_opt in (current_opt.deprecated_opts or [])
+}
 
 _auth_plugin = None
 _session = None
@@ -54,16 +73,15 @@ class SessionWithAccessTokenRefresh(session.Session):
 
     def __init__(self, auth=None, **kwargs):
         def get_access_token(_auth):
-            expires_at = getattr(_auth, '_expires_at', time.time())
-            should_refresh = (
-                expires_at - time.time() < self.REFRESH_THRESHOLD)
-            if not getattr(_auth, '_access_token', None) or should_refresh:
+            expires_at = getattr(_auth, "_expires_at", time.time())
+            should_refresh = expires_at - time.time() < self.REFRESH_THRESHOLD
+            if not getattr(_auth, "_access_token", None) or should_refresh:
                 try:
                     access_token, expires_at = jupyterhub.refresh_access_token()
                     _auth._access_token = access_token
                     _auth._expires_at = expires_at
                 except Exception as e:
-                    LOG.error('Failed to refresh access_token: %s', e)
+                    LOG.error("Failed to refresh access_token: %s", e)
             return _auth._access_token
 
         def set_access_token(_auth, access_token):
@@ -82,13 +100,15 @@ class SessionWithAccessTokenRefresh(session.Session):
         # Session.request function is not enough because there are many other
         # points at which an authentication is performed (e.g., looking up an
         # endpoint.)
-        if (auth and isinstance(auth, OidcAccessToken)
-            and jupyterhub.is_jupyterhub_env()):
+        if (
+            auth
+            and isinstance(auth, OidcAccessToken)
+            and jupyterhub.is_jupyterhub_env()
+        ):
             auth._access_token = None
             auth._expires_at = 0
             # Monkey-patch access_token to be @property dynamic lookup
-            auth.__class__.access_token = (
-                property(get_access_token, set_access_token))
+            auth.__class__.access_token = property(get_access_token, set_access_token)
 
         super().__init__(auth=auth, **kwargs)
 
@@ -101,13 +121,15 @@ def _auth_plugins():
     return [
         (name, [o._to_oslo_opt() for o in loader.get_options()])
         for name, loader in loading.get_available_plugin_loaders().items()
-        if name.startswith('v3')
+        if name.startswith("v3")
     ]
 
 
 def _default_from_env(opts, group=None):
     def _default(opt):
-        all_opts = [opt] + (getattr(opt, 'deprecated_opts', getattr(opt, 'deprecated', None)) or [])
+        all_opts = [opt] + (
+            getattr(opt, "deprecated_opts", getattr(opt, "deprecated", None)) or []
+        )
         for o in all_opts:
             v = os.environ.get(f'OS_{o.name.replace("-", "_").upper()}')
             if v:
@@ -123,7 +145,7 @@ def _default_from_env(opts, group=None):
 
 
 def _auth_section(auth_plugin):
-    return f'{CONF_GROUP}.auth.{auth_plugin}'
+    return f"{CONF_GROUP}.auth.{auth_plugin}"
 
 
 def _set_auth_plugin(auth_plugin):
@@ -139,6 +161,16 @@ def _set_auth_plugin(auth_plugin):
     _auth_plugin = auth_plugin
 
 
+def _check_deprecated(key):
+    if key not in deprecated_extra_opts:
+        return key
+
+    LOG.warning(
+        f"Option '{key}' is deprecated; please use '{deprecated_extra_opts[key]}' instead."
+    )
+    return deprecated_extra_opts[key]
+
+
 def set(key, value):
     """Set a context parameter by name.
 
@@ -152,9 +184,11 @@ def set(key, value):
     global _session
     # Special handling for auth_type setting as we have to also tell KSA to
     # start reading auth parameters from the plugin's auth section.
+
     if key == _AUTH_TYPE_OPT.dest:
         _set_auth_plugin(value)
-    elif key in [o.dest for o in global_options]:
+    elif key in global_option_names:
+        key = _check_deprecated(key)
         cfg.CONF.set_override(key, value, group=CONF_GROUP)
     else:
         valid_for_some_plugin = False
@@ -184,7 +218,9 @@ def get(key):
         cfg.NoSuchOptError: if the parameter is not supported.
     """
     global _auth_plugin
-    if key in [o.dest for o in global_options]:
+
+    if key in global_option_names:
+        key = _check_deprecated(key)
         return cfg.CONF[CONF_GROUP][key]
     else:
         return cfg.CONF[_auth_section(_auth_plugin)][key]
@@ -228,40 +264,45 @@ def use_site(site_name):
     """
     global _sites
     if not _sites:
-        res = requests.get(f'{RESOURCE_API_URL}/sites.json')
+        res = requests.get(f"{RESOURCE_API_URL}/sites.json")
         try:
             res.raise_for_status()
-            items = res.json().get('items', [])
-            _sites = {s['name']: s for s in items}
+            items = res.json().get("items", [])
+            _sites = {s["name"]: s for s in items}
             if not _sites:
-                raise ValueError('No sites returned.')
+                raise ValueError("No sites returned.")
         except Exception:
-            printerr("""Failed to fetch list of available Chameleon sites.
+            printerr(
+                """Failed to fetch list of available Chameleon sites.
                 You can still set the site information manually like this,
                 if you know the URL and name:
 
                     chi.set('auth_url', 'https://chi.uc.chameleoncloud.org:5000/v3')
                     chi.set('region_name', 'CHI@UC')
-                """)
+                """
+            )
             return
 
     site = _sites.get(site_name)
     if not site:
-        raise ValueError((
-            f'No site named "{site_name}" exists! Possible values: '
-            ', '.join(_sites.keys())))
+        raise ValueError(
+            (
+                f'No site named "{site_name}" exists! Possible values: '
+                ", ".join(_sites.keys())
+            )
+        )
 
     # Set important parameters
-    set('auth_url', f'{site["web"]}:5000/v3')
-    set('region_name', site['name'])
+    set("auth_url", f'{site["web"]}:5000/v3')
+    set("region_name", site["name"])
 
     output = [
-        f'Now using {site_name}:',
+        f"Now using {site_name}:",
         f'URL: {site.get("web")}',
         f'Location: {site.get("location")}',
         f'Support contact: {site.get("user_support_contact")}',
     ]
-    print('\n'.join(output))
+    print("\n".join(output))
 
 
 def session():
@@ -274,7 +315,9 @@ def session():
     if not _session:
         auth = loading.load_auth_from_conf_options(cfg.CONF, CONF_GROUP)
         sess = SessionLoader().load_from_conf_options(cfg.CONF, CONF_GROUP, auth=auth)
-        _session = loading.load_adapter_from_conf_options(cfg.CONF, CONF_GROUP, session=sess)
+        _session = loading.load_adapter_from_conf_options(
+            cfg.CONF, CONF_GROUP, session=sess
+        )
     return _session
 
 
@@ -293,8 +336,8 @@ def reset():
     _sites = {}
     cfg.CONF.reset()
     _set_auth_plugin(
-        os.getenv('OS_AUTH_TYPE',
-            os.getenv('OS_AUTH_METHOD', DEFAULT_AUTH_TYPE)))
+        os.getenv("OS_AUTH_TYPE", os.getenv("OS_AUTH_METHOD", DEFAULT_AUTH_TYPE))
+    )
     _default_from_env(global_options, group=CONF_GROUP)
     for auth_plugin, opts in _auth_plugins():
         _default_from_env(opts, group=_auth_section(auth_plugin))
